@@ -571,16 +571,29 @@ rb_playlist_manager_save_thread_main (struct RBPlaylistManagerSaveThreadData *da
 	return NULL;
 }
 
+static void
+save_source_iter_to_xml (GtkTreeModel *model, GtkTreeIter *iter,
+			 xmlNodePtr root)
+{
+	RBSource *source;
+	gtk_tree_model_get (model, iter, RB_SOURCELIST_MODEL_COLUMN_SOURCE,
+			    &source, -1);
+	if (RB_IS_PLAYLIST_SOURCE (source))
+		rb_playlist_source_save_to_xml (RB_PLAYLIST_SOURCE (source),
+						root);
+}
+
 void
-rb_playlist_manager_save_playlists (RBPlaylistManager *mgr)
+rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
 {
-	GList *tmp;
 	xmlNodePtr root;
 	struct RBPlaylistManagerSaveThreadData *data;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
 
 	rb_debug ("saving the playlists");
 
-	if (!mgr->priv->dirty) {
+	if (!force && !mgr->priv->dirty) {
 		rb_debug ("no save needed, ignoring");
 		return;
 	}
@@ -600,9 +613,13 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr)
 
 	root = xmlNewDocNode (data->doc, NULL, "rhythmdb-playlists", NULL);
 	xmlDocSetRootElement (data->doc, root);
-	
-	for (tmp = mgr->priv->playlists; tmp; tmp = tmp->next)
-		rb_playlist_source_save_to_xml (RB_PLAYLIST_SOURCE (tmp->data), root);
+
+	model = rb_sourcelist_get_model (mgr->priv->sourcelist);
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		do {
+			save_source_iter_to_xml (model, &iter, root);
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
 
 	g_object_ref (G_OBJECT (mgr));
 	mgr->priv->outstanding_threads++;
@@ -662,7 +679,7 @@ rb_playlist_manager_set_automatic_playlist (RBPlaylistManager *mgr,
 	rb_query_creator_get_limit (creator, &type, &limit);
 	if (type == RB_QUERY_CREATOR_LIMIT_COUNT)
 		limit_count = limit;
-	if (type == RB_QUERY_CREATOR_LIMIT_MB)
+	else if (type == RB_QUERY_CREATOR_LIMIT_MB)
 		limit_size = limit;
 	rb_playlist_source_set_query (playlist,
 				      rb_query_creator_get_query (creator),
@@ -685,7 +702,6 @@ rb_playlist_manager_cmd_new_automatic_playlist (BonoboUIComponent *component,
 		return;
 	}
 
-
 	playlist = rb_playlist_manager_new_playlist (mgr, NULL, TRUE);
 
 	rb_playlist_manager_set_automatic_playlist (mgr, RB_PLAYLIST_SOURCE (playlist), creator); 
@@ -790,7 +806,11 @@ save_playlist_response_cb (GtkDialog *dialog,
 		return;
 	}
 
+#ifndef HAVE_GTK_2_3
 	file = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog)));
+#else
+	file = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+#endif
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
